==> libremail-2.2.3.tar.bz2: libremail/src/chargechampok.c <==
/*
    Fichier chargemail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme tlcharge parmi les mail en attente sur le serveur
    de messagerie, ceux dont un champ choisi contient une chaine de
    caractres autorise.

    La liste des chaines de caractres autorise pour ce champ est
    mmorise dans un fichier.

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres et pour fixer le rpertoire racine du systme
    de messagerie.

    Les mail entrants sont stocks dans le sous rpertoire "entree"
    de ce rpertoire racine et se distinguent par leur numro.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "numail.h"
#include "testchamp.h"
#include "trtentete.h"
#include "szchemin.h"


/* constantes pour indiquer si l'on efface le(s) mail(s) tlcharg(s) */
#define OPTDEFAUT  0
#define EFFACE     1
#define CONSERVE   2


/* prototypes */
int testchamp (int numes, char *nomchamp);
void chargemail (int numes, long numail, int effacement);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

char **listechamp;        // mmorise les champs interdits
int  sz_listechamp;       // nombre de champs mmoriss dans listechamp
char bufChamp [80];       // buffer contenant le champ du mail  tester
char dirmails [szchemin]; // rpertoire de destination des messages

// chaines de caractres mmorises pour viter appels rptitifs  message ()
char mess_analys [50]; // message signalant l'analyse d'un mail
char mess_telech [50]; // message signalant le tlchargement d'un mail



/* programme principal */

int main (int nbarg, char *varg[])
{
    FILE *fconf;         // descripteur du fichier de configuration
    char fichliste [szchemin]; // fichier contenant liste des chaines  tester
    char *nomchamp;      // nom du champ  tester
    char *nomfliste;     // nom du fichier contenant les chaines  tester
    int  numes, nbmes;   // numro du mail courant et nombre de mails
    int  nbrecup = 0;    // nombre de mails tlchargs
    long numail;         // numro du mail dans le fichier tlcharg
    int  option;  // dtermine si on conserve sur le serveur les mails rcuprs
    int  i, j;           // compteurs


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de chargechampoc
    nbarg --;
    varg ++;

    // rcupration du nom du champ  tester
    if (nbarg > 1)
    {
        nomchamp = varg [0];
        nbarg --;
        varg ++;
    }

    // initialisations valeur des options
    nomfliste = NULL;
    option = OPTDEFAUT;

    // si le programme a t lanc avec des options
    while (nbarg >= 2 && **varg == '-')
    {
        switch (varg [0][1])
        {
            // option effacement du serveur des mails rcuprs
            case 'd': if (option == CONSERVE)
                      {
                          // "Options -d et -k incompatibles"
                          affiche_err ("COMPAT_OPT_DK");
                          return (-1);
                      }
                      else
                          option = EFFACE;

                      break;

            // option conservation sur le serveur des mails rcuprs
            case 'k': if (option == EFFACE)
                      {
                          // "Options -d et -k incompatibles"
                          affiche_err ("COMPAT_OPT_DK");
                          return (-1);
                      }
                      else
                          option = CONSERVE;

                      break;

            // option nom du fichier contenant la liste des chaines autorises
            case 'f' : nomfliste = varg [1];

                       // dcompter l'argument supplmentaire trait
                       varg ++;
                       nbarg --;
                       break;

                       // "Option %s incorrecte"
            default  : aff_err_arg ("ERR_OPTION", *varg);
                       nbarg = 0;   // pour afficher la syntaxe de la commande
        }

        // dcompter l'argument trait
        varg ++;
        nbarg --;
    }

    // controle du nombre d'arguments
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [0]);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer le rpertoire racine de la messagerie
                fgets (dirmails, sizeof (dirmails) - 7, fconf);
                dirmails [strlen (dirmails) - 1] = '\0';

                // si ce rpertoire existe
                if (access (dirmails, 0) == 0)
                {
                    // dtermination du nom du fichier
                    // contenant les chaines interdites

                    // cas particulier : chemin d'accs absolu  ce fichier
                    if (nomfliste && *nomfliste == '/')
                        strcpy (fichliste, nomfliste);
                    else
                    {
                        // cas gnral : pas de chemin d'accs absolu au fichier

                        // on complte ce nom d'un /
                        strcpy (fichliste, dirmails);
                        strcat (fichliste, "/");

                        // si le nom de ce fichier a t pass en paramtre
                        if (nomfliste)
                            // le rajouter au nom du rpertoire
                            strcat (fichliste, nomfliste);
                        else
                        {
                            // sinon, on va gnrer un nom 
                            // partir du nom du champ
                            strcat (fichliste, ficdir ("FIC_ACCEPT_"));

                            i = strlen (fichliste);
                            j = 0;

                            do
                                // le nom du champ est recopi en minuscules
                                fichliste [i++] = tolower (nomchamp [j]);
                            while (nomchamp [j++]);
                        }
                    }

                    // charger en mmoire les champs interdits
                    listechamp = charge_valchamp (fichliste, &sz_listechamp);

                    // si la liste n'est pas vide
                    if (sz_listechamp)
                    {
                        // crer si ncessaire le sous repertoire d'entre
                        strcat (dirmails, "/");
                        strcat (dirmails, ficdir ("DIR_ENTREE"));
                        mkdir (dirmails, 0755);

                        // rcupration du nombre de mails
                        nbmes = nbmails ();

                        // si des mails peuvent tre rcuprs
                        if (nbmes)
                        {
                            // rcupration du numro du dernier fichier mail
                            numail = num_dermail (dirmails);

                            // Initialisation des messages  afficher  chaque
                            // analyse ou tlchargement de mail

                            // "\rAnalyse du mail n %d       "
                            strcpy (mess_analys, message ("ANALYSE_MAIL_BL"));

                            // "\rTlchargement du mail n %d"
                            strcpy (mess_telech, message ("TELECH_MAIL"));

                            // pour chaque fichier mail du serveur
                            for (numes = 1; numes <= nbmes; numes++)
                            {
                                // si l'expditeur du mail est autoris
                                if (testchamp (numes, nomchamp))
                                {
                                    // tlcharger le mail correspondant
                                    chargemail (numes, ++numail, option);

                                    // on compte les mails tlchargs
                                    nbrecup ++;
                                }
                            }

                            // dition d'un rcapitulatif
                            // "\n%d mails tlchargs sur %d\n"
                            printf (message ("TOTAL_TELECH"), nbrecup, nbmes);

                            // sauvegarde du numro du dernier fichier mail
                            sauv_num_dermail (dirmails, numail);
                        }
                    }
                }
                else
                    // "rpertoire de messagerie %s inexistant"
                    aff_err_arg ("REP_RACINE_ABSENT", dirmails);

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        // "Syntaxe : %s nom_champ [-f fich_liste] [-(k|d)] fich_configuration"
        psyntaxe ("SYNT_CHARGECHAMPOK");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}




/* lit l'entte d'un mail et vrifie si son champ est autoris */

int testchamp (int numes, char *nomchamp)
{
    char bufw [80];       // buffer d'envoi d'une requte
    int  i;               // simple compteur


    // "\rAnalyse du mail n %d       "
    printf (mess_analys, numes);
    fflush (stdout);

    // initialisation
    bufChamp [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture entte du message et mmorisation de le champ du mail
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si on a trouv le champ choisi
        if (start (nomchamp))
        {
            // convertir les caractres spciaux de la ligne
            majlignentete ();

            // initialisation
            i = strlen (nomchamp) + 1;

            // recherche du dbut du champ
            while (buf_lect [i] == ' ')
                i++;

            // tronquer si ncessaire le champ (pour viter un plantage)
            if (strlen (buf_lect + i) > sizeof (bufChamp))
                 buf_lect [i + sizeof (bufChamp) - 1] = '\0';

            // mmorisation du champ
            strcpy (bufChamp, buf_lect + i);
        }
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // vrifier si le mail contient un des champ de la liste interdite
    return trouve_valchamp (bufChamp, listechamp, sz_listechamp);
}



/* tlcharge le mail choisi et l'enlve du serveur si c'est demand */

void chargemail (int numes, long numail, int effacement)
{
    char bufw [80];    // buffer d'envoi d'une requte
    char ficmail [sizeof (dirmails) + 9];
    FILE *fmail;


    // cration du fichier mail
    sprintf (ficmail, "%s/r%07ld.n", dirmails, numail);

    fmail = fopen (ficmail, "w");

    if (fmail)
    {
        // message de suivi du droulement
        // "\rTlchargement du mail n %d"
        printf (mess_telech, numes);
        fflush (stdout);

        // demande de lecture du message et rception de l'acquittement
        sprintf (bufw, "RETR %d", numes);
        env_pop (bufw);
        lire_pop ();

        // lire la premire ligne de l'entte du mail
        lire_pop ();

        // lecture et copie du message jusqu' une ligne limite a un .
        while (buf_lect [0] != '.' || buf_lect [1] != '\0')
        {
            // copie de la ligne lue dans le fichier mail
            fputs (buf_lect, fmail);
            fputc ('\n', fmail);     // ncessaire car supprim de buf_lect

            // et lecture de la suivante
            lire_pop ();
        }

        // fermer le fichier contenant le mail
        fclose (fmail);

        // et supprimer si ncessaire le mail du serveur
        if (effacement != CONSERVE)
        {
            sprintf (bufw, "DELE %d", numes);
            env_pop (bufw);
            lire_pop ();
        }
    }
    else
        // "Impossible d'crire dans le fichier %s"
        aff_err_arg ("IMPOS_ECR_FICH", ficmail);
}

==> libremail-2.2.4.tar.bz2: libremail/src/chargechampok.c <==
/*
    Fichier chargemail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme tlcharge parmi les mail en attente sur le serveur
    de messagerie, ceux dont un champ choisi contient une chaine de
    caractres autorise.

    La liste des chaines de caractres autorise pour ce champ est
    mmorise dans un fichier.

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres et pour fixer le rpertoire racine du systme
    de messagerie.

    Les mail entrants sont stocks dans le sous rpertoire "entree"
    de ce rpertoire racine et se distinguent par leur numro.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "numail.h"
#include "testchamp.h"
#include "trtentete.h"
#include "szchemin.h"


/* constantes pour indiquer si l'on efface le(s) mail(s) tlcharg(s) */
#define OPTDEFAUT  0
#define EFFACE     1
#define CONSERVE   2


/* prototypes */
int testchamp (int numes, char *nomchamp);
void chargemail (int numes, long numail, int effacement);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

char **listechamp;        // mmorise les champs interdits
int  sz_listechamp;       // nombre de champs mmoriss dans listechamp
char bufChamp [80];       // buffer contenant le champ du mail  tester
char dirmails [szchemin]; // rpertoire de destination des messages

// chaines de caractres mmorises pour viter appels rptitifs  message ()
char mess_analys [50]; // message signalant l'analyse d'un mail
char mess_telech [50]; // message signalant le tlchargement d'un mail



/* programme principal */

int main (int nbarg, char *varg[])
{
    FILE *fconf;         // descripteur du fichier de configuration
    char fichliste [szchemin]; // fichier contenant liste des chaines  tester
    char *nomchamp;      // nom du champ  tester
    char *nomfliste;     // nom du fichier contenant les chaines  tester
    int  numes, nbmes;   // numro du mail courant et nombre de mails
    int  nbrecup = 0;    // nombre de mails tlchargs
    long numail;         // numro du mail dans le fichier tlcharg
    int  option;  // dtermine si on conserve sur le serveur les mails rcuprs
    int  i, j;           // compteurs


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de chargechampoc
    nbarg --;
    varg ++;

    // rcupration du nom du champ  tester
    if (nbarg > 1)
    {
        nomchamp = varg [0];
        nbarg --;
        varg ++;
    }

    // initialisations valeur des options
    nomfliste = NULL;
    option = OPTDEFAUT;

    // si le programme a t lanc avec des options
    while (nbarg >= 2 && **varg == '-')
    {
        switch (varg [0][1])
        {
            // option effacement du serveur des mails rcuprs
            case 'd': if (option == CONSERVE)
                      {
                          // "Options -d et -k incompatibles"
                          affiche_err ("COMPAT_OPT_DK");
                          return (-1);
                      }
                      else
                          option = EFFACE;

                      break;

            // option conservation sur le serveur des mails rcuprs
            case 'k': if (option == EFFACE)
                      {
                          // "Options -d et -k incompatibles"
                          affiche_err ("COMPAT_OPT_DK");
                          return (-1);
                      }
                      else
                          option = CONSERVE;

                      break;

            // option nom du fichier contenant la liste des chaines autorises
            case 'f' : nomfliste = varg [1];

                       // dcompter l'argument supplmentaire trait
                       varg ++;
                       nbarg --;
                       break;

                       // "Option %s incorrecte"
            default  : aff_err_arg ("ERR_OPTION", *varg);
                       nbarg = 0;   // pour afficher la syntaxe de la commande
        }

        // dcompter l'argument trait
        varg ++;
        nbarg --;
    }

    // controle du nombre d'arguments
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [0]);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer le rpertoire racine de la messagerie
                fgets (dirmails, sizeof (dirmails) - 7, fconf);
                dirmails [strlen (dirmails) - 1] = '\0';

                // si ce rpertoire existe
                if (access (dirmails, 0) == 0)
                {
                    // dtermination du nom du fichier
                    // contenant les chaines interdites

                    // cas particulier : chemin d'accs absolu  ce fichier
                    if (nomfliste && *nomfliste == '/')
                        strcpy (fichliste, nomfliste);
                    else
                    {
                        // cas gnral : pas de chemin d'accs absolu au fichier

                        // on complte ce nom d'un /
                        strcpy (fichliste, dirmails);
                        strcat (fichliste, "/");

                        // si le nom de ce fichier a t pass en paramtre
                        if (nomfliste)
                            // le rajouter au nom du rpertoire
                            strcat (fichliste, nomfliste);
                        else
                        {
                            // sinon, on va gnrer un nom 
                            // partir du nom du champ
                            strcat (fichliste, ficdir ("FIC_ACCEPT_"));

                            i = strlen (fichliste);
                            j = 0;

                            do
                                // le nom du champ est recopi en minuscules
                                fichliste [i++] = tolower (nomchamp [j]);
                            while (nomchamp [j++]);
                        }
                    }

                    // charger en mmoire les champs interdits
                    listechamp = charge_valchamp (fichliste, &sz_listechamp);

                    // si la liste n'est pas vide
                    if (sz_listechamp)
                    {
                        // crer si ncessaire le sous repertoire d'entre
                        strcat (dirmails, "/");
                        strcat (dirmails, ficdir ("DIR_ENTREE"));
                        mkdir (dirmails, 0755);

                        // rcupration du nombre de mails
                        nbmes = nbmails ();

                        // si des mails peuvent tre rcuprs
                        if (nbmes)
                        {
                            // rcupration du numro du dernier fichier mail
                            numail = num_dermail (dirmails);

                            // Initialisation des messages  afficher  chaque
                            // analyse ou tlchargement de mail

                            // "\rAnalyse du mail n %d       "
                            strcpy (mess_analys, message ("ANALYSE_MAIL_BL"));

                            // "\rTlchargement du mail n %d"
                            strcpy (mess_telech, message ("TELECH_MAIL"));

                            // pour chaque fichier mail du serveur
                            for (numes = 1; numes <= nbmes; numes++)
                            {
                                // si l'expditeur du mail est autoris
                                if (testchamp (numes, nomchamp))
                                {
                                    // tlcharger le mail correspondant
                                    chargemail (numes, ++numail, option);

                                    // on compte les mails tlchargs
                                    nbrecup ++;
                                }
                            }

                            // dition d'un rcapitulatif
                            // "\n%d mails tlchargs sur %d\n"
                            printf (message ("TOTAL_TELECH"), nbrecup, nbmes);

                            // sauvegarde du numro du dernier fichier mail
                            sauv_num_dermail (dirmails, numail);
                        }
                    }
                }
                else
                    // "rpertoire de messagerie %s inexistant"
                    aff_err_arg ("REP_RACINE_ABSENT", dirmails);

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        // "Syntaxe : %s nom_champ [-f fich_liste] [-(k|d)] fich_configuration"
        psyntaxe ("SYNT_CHARGECHAMPOK");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}




/* lit l'entte d'un mail et vrifie si son champ est autoris */

int testchamp (int numes, char *nomchamp)
{
    char bufw [80];       // buffer d'envoi d'une requte
    int  i;               // simple compteur


    // "\rAnalyse du mail n %d       "
    printf (mess_analys, numes);
    fflush (stdout);

    // initialisation
    bufChamp [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lire la premire ligne de l'entte du message
    lire_pop ();

    // termin pour ce mail si erreur d'envoi cot serveur
    if (memcmp (buf_lect, "-ERR ", 5) == 0)
    {
        // Erreur serveur pour l'accs au mail
        aff_err_argnum ("ERREUR_SERVEUR", numes);
        return 0;
    }

    // lecture entte du message et mmorisation de le champ du mail
    do
    {
        if (start (nomchamp))
        {
            // convertir les caractres spciaux de la ligne
            majlignentete ();

            // initialisation
            i = strlen (nomchamp) + 1;

            // recherche du dbut du champ
            while (buf_lect [i] == ' ')
                i++;

            // tronquer si ncessaire le champ (pour viter un plantage)
            if (strlen (buf_lect + i) > sizeof (bufChamp))
                 buf_lect [i + sizeof (bufChamp) - 1] = '\0';

            // mmorisation du champ
            strcpy (bufChamp, buf_lect + i);
        }

        // lire la ligne suivante de l'entte du message
        lire_pop ();
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // vrifier si le mail contient un des champ de la liste interdite
    return trouve_valchamp (bufChamp, listechamp, sz_listechamp);
}



/* tlcharge le mail choisi et l'enlve du serveur si c'est demand */

void chargemail (int numes, long numail, int effacement)
{
    char bufw [80];    // buffer d'envoi d'une requte
    char ficmail [sizeof (dirmails) + 9];
    FILE *fmail;


    // cration du fichier mail
    sprintf (ficmail, "%s/r%07ld.n", dirmails, numail);

    fmail = fopen (ficmail, "w");

    if (fmail)
    {
        // message de suivi du droulement
        // "\rTlchargement du mail n %d"
        printf (mess_telech, numes);
        fflush (stdout);

        // demande de lecture du message et rception de l'acquittement
        sprintf (bufw, "RETR %d", numes);
        env_pop (bufw);
        lire_pop ();

        // termin pour ce mail si erreur d'envoi cot serveur
        if (memcmp (buf_lect, "-ERR ", 5) == 0)
        {
            // Erreur serveur pour l'accs au mail
            aff_err_argnum ("ERREUR_SERVEUR", numes);
            fclose (fmail);
            unlink (ficmail);
            return;
        }

        // lire la premire ligne de l'entte du mail
        lire_pop ();

        // lecture et copie du message jusqu' une ligne limite a un .
        while (buf_lect [0] != '.' || buf_lect [1] != '\0')
        {
            // copie de la ligne lue dans le fichier mail
            fputs (buf_lect, fmail);
            fputc ('\n', fmail);     // ncessaire car supprim de buf_lect

            // et lecture de la suivante
            lire_pop ();
        }

        // fermer le fichier contenant le mail
        fclose (fmail);

        // et supprimer si ncessaire le mail du serveur
        if (effacement != CONSERVE)
        {
            sprintf (bufw, "DELE %d", numes);
            env_pop (bufw);
            lire_pop ();
        }
    }
    else
        // "Impossible d'crire dans le fichier %s"
        aff_err_arg ("IMPOS_ECR_FICH", ficmail);
}
